//
// k b c . c
//
// Keyboard controller module
//
// Sep. 10, 2002 by W.Nishida (http://www.skyfree.org)
//

#include "kbc.h"

asm(".code16gcc");

// Input data from specified port
//	port: target port number

int inb(int port) {
  asm("
	pushl	%edx			# Save current EDX
	xorl	%eax, %eax
        movl    8( %ebp ), %edx		# Get port address
        inb     %dx, %al		# Input data
	popl	%edx			# Restore EDX
  ");					// EAX holds data
 }

// Output data to specified port
//	data: data to be sent
//	port: target port number

void outb(int data, int port) {
  asm("
	pushl	%edx			# Save current EDX
        movl    12( %ebp ), %edx	# Get port address
        movl    8( %ebp ), %eax		# Get data to be written
        outb    %al, %dx		# Output data
	popl	%edx			# Restore EDX
  ");
 }

// Read the status register of 8042 KBC
//      return: data

int kbc_status(void) {
  return( inb(KBC_CTRL) );	// Read status register of 8042
 }

// Read the output data buffer of 8042
//	return: data

int kbc_read(void) {

  while (1) {				// Wait until output data is available
    if (kbc_status() & KBC_OBF)		// Is 8042 output data buffer ready?
      break;				// Then exit loop
    inb(IOWAIT);			// Do wait
   }
  inb(IOWAIT);				// Do wait
  return( inb(KBC_DATA) );		// Read 8042 output data buffer
 }

// Disable keyboard interrupt (IRQ1)

void disable_keyint(void) {
  int imr;

  imr = inb(PIC_MIMR);		// Read current master IMR
  imr |= PIC_IMR_KEY;		// Mask IRQ1 bit
  outb(imr, PIC_MIMR);		// Update IMR
 }

// Enable keyboard interrupt (IRQ1)

void enable_keyint(void) {
  int imr;

  imr = inb(PIC_MIMR);		// Read current master IMR
  imr &= ~PIC_IMR_KEY;		// Unmask IRQ1 bit
  outb(imr, PIC_MIMR);		// Update IMR
 }
